//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#include "ReadTriggerSegments.h"

ClassImp(ReadTriggerSegments)

////////////////////////////////////////////////////////////////////////////////////
ReadTriggerSegments::ReadTriggerSegments(const string aPattern, const string aDirectory, const int aVerbose){ 
////////////////////////////////////////////////////////////////////////////////////
  
  // parameters
  Verbose=aVerbose;  // set verbosity level
  Pattern=aPattern;  // file pattern
  Dir=aDirectory;    // directory
  status_OK=true;
  
  // nulling
  Seg    = new Segments();
  Tstart = new double[0];
  Tstop  = new double[0];
                                
  // random id
  srand(time(NULL));
  randid = rand();
  ostringstream tmpstream;
  tmpstream<<randid;
  srandid = tmpstream.str();

  // get segments
  status_OK*=GetInputSegments();
}

////////////////////////////////////////////////////////////////////////////////////
ReadTriggerSegments::~ReadTriggerSegments(void){
////////////////////////////////////////////////////////////////////////////////////
  if(Verbose>1) cout<<"ReadTriggerSegments::~ReadTriggerSegments"<<endl;
  delete Tstart;
  delete Tstop;
  delete Seg;
  triggerfilename.clear();
}

////////////////////////////////////////////////////////////////////////////////////
bool ReadTriggerSegments::GetInputSegments(void){ 
////////////////////////////////////////////////////////////////////////////////////

  // clear
  triggerfilename.clear();
  Seg->Reset();

  // chain segment trees...
  TChain *SegTree;
  if(Dir.compare("")) SegTree= new TChain((Dir+"/segments").c_str());
  else                SegTree= new TChain("segments");

  // get list of patterns and load it
  vector <string> patlist = SplitString(Pattern,' ');
  for(int p=0; p<(int)patlist.size(); p++) SegTree->Add(patlist[p].c_str());
  SegTree->SetName(("segments_"+srandid).c_str());
  patlist.clear();

  // consistency check
  if(!SegTree->GetNtrees()){// no data
    cerr<<"ReadTriggerSegments::GetInputSegments: no trigger files"<<endl;
    delete SegTree;
    return false;
  }
  if(!SegTree->GetEntries()){// empty tree
    cerr<<"ReadTriggerSegments::GetInputSegments: no data"<<endl;
    delete SegTree;
    return false;
  }
  if(!SegTree->GetBranchStatus("start")){
    cerr<<"ReadTriggerSegments::GetInputSegments: missing start branch"<<endl;
    delete SegTree;
    return false;
  }
  if(!SegTree->GetBranchStatus("end")){
    cerr<<"ReadTriggerSegments::GetInputSegments: missing end branch"<<endl;
    delete SegTree;
    return false;
  }

  // number of input files
  int Ninfiles=SegTree->GetNtrees();

  // allocate memory for each files 
  delete Tstart; Tstart = new double [Ninfiles];
  delete Tstop; Tstop  = new double [Ninfiles];
  
  // mapping the files
  if(Verbose>1) cout<<"ReadTriggerSegments::GetInputSegments: mapping the trigger files"<<endl;
  double s; SegTree->SetBranchAddress("start",&s);// this is just for GetEntry()
  double e; SegTree->SetBranchAddress("end",&e);  // this is just for GetEntry()
  for(int i=0; i<Ninfiles; i++){
    SegTree->GetEntry(SegTree->GetTreeOffset()[i]);// load tree
    Tstart[i] = SegTree->GetTree()->GetMinimum("start");
    Tstop[i] = SegTree->GetTree()->GetMaximum("end");
    triggerfilename.push_back(SegTree->GetFile()->GetPath());
          
    // check that files are time sorted
    if(i&&Tstart[i]<Tstop[i-1]){
      cerr<<"ReadTriggerSegments::GetInputSegments: the input files are not time-sorted in the file pattern: this class cannot work! ("<<Tstart[i]<<"<"<<Tstop[i-1]<<")"<<endl;
      delete SegTree;
      return false;
    }
  }

  // get segments
  if(Verbose) cout<<"ReadTriggerSegments::GetInputSegments: extract segments from trees"<<endl;
  for(int i=0; i<SegTree->GetEntries(); i++){
    SegTree->GetEntry(i);
    Seg->seg_start.push_back(s);
    Seg->seg_end.push_back(e);
    Seg->seg_tag.push_back(1);
  }
  status_OK*=Seg->CheckConsistency();
  status_OK*=Seg->MergeOverlaps(s);
  status_OK*=Seg->SetMarks();
  status_OK*=Seg->ComputeLiveTime();
  delete SegTree;
      
  return status_OK;
}

////////////////////////////////////////////////////////////////////////////////////
string ReadTriggerSegments::GetTriggerFiles(const double aTimeMin, const double aTimeMax){ 
////////////////////////////////////////////////////////////////////////////////////
  if(!status_OK){
    cerr<<"ReadTriggerSegments::GetTriggerFiles: the ReadTriggerSegments object is corrupted"<<endl;
    return "none";
  }

  // get list of files
  string slist="";
  for(int i=0; i<(int)triggerfilename.size(); i++){
    if(Tstop[i]<=aTimeMin) continue;
    if(Tstart[i]>=aTimeMax) break;
    slist+=triggerfilename[i]+" ";
  }
 
  if(slist.compare("")) return slist;
  return "none";
}

////////////////////////////////////////////////////////////////////////////////////
string ReadTriggerSegments::GetTriggerFiles(Segments *aSegSelect){ 
////////////////////////////////////////////////////////////////////////////////////
  if(!status_OK){
    cerr<<"ReadTriggerSegments::GetTriggerFiles: the ReadTriggerSegments object is corrupted"<<endl;
    return "none";
  }

  // get list of files
  string slist="";
  ReadTriggerSegments *segfile;
  for(int i=0; i<(int)triggerfilename.size(); i++){
    segfile = new ReadTriggerSegments(triggerfilename[i]);
    segfile->Seg->Intersect(aSegSelect);
    if(segfile->Seg->GetLiveTime()) slist+=triggerfilename[i]+" ";
    delete segfile;
  }
  
  if(slist.compare("")) return slist;
  return "none";
}

