//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#ifndef __TriggerPlot__
#define __TriggerPlot__

#include "GwollumPlot.h"
#include "ReadTriggers.h"
#include "TriggerSelect.h"
#include "Date.h"
#include "TF1.h"
#include "TH1.h"
#include "TH2.h"
#include "TGraph.h"

using namespace std;

/**
 * Plot triggers.
 * This class was designed to produce and display a 'collection' of plots representing a set of triggers. A collection is composed of several plot types:
 * - "rate": trigger rate as a function of time
 * - "frequency": frequency distribution
 * - "snr": SNR distribution
 * - "freqtime": frequency versus time
 * - "snrtime": SNR vs time
 * - "snrfreq": SNR vs frequency
 *
 * A collection is defined by a 'selection' which is a set of criteria used to select the triggers to be plotted. This way, several collections can be defined and plotted in the same time.
 * \author Florent Robinet
 */
class TriggerPlot: public GwollumPlot, public ReadTriggers {

 public:

  /**
   * @name Constructors and destructors
   @{
  */
  /**
   * Constructor of the TriggerPlot class.
   * A set of 'aNcoll' collections of plots are initialized. They will be filled with the input triggers. See ReadTriggers::ReadTriggers() to know how to input triggers. By default, the plot settings are defined using the input trigger metadata and are the same for all collections.
   * @param aNcoll number of collections
   * @param aPattern input file pattern
   * @param aDirectory trigger ROOT directory
   * @param aVerbose verbosity level
   */
  TriggerPlot(const int aNcoll, const string aPattern, const string aDirectory="", const string aStyleName="GWOLLUM", const int aVerbose=0);

  /**
   * Destructor of the TriggerPlot class.
   */
  virtual ~TriggerPlot(void);
  /**
     @}
  */

  /**
   * Builds collection plots.
   * Once the selections are made, use this function to build all the plots for all collections.
   * The argument of this function can be used to make a single collection instead of all. 
   * 
   * If an input Segments object is provided, the segment bar will include this constraint
   * @param aCollIndex collection index (if <0 --> make all)
   * @param aInSeg input segments
   */
  bool MakeCollections(const int aCollIndex=-1, Segments *aInSeg=NULL);

  /**
   * Prints an external plot.
   */
  inline void PrintPlot(TObject *Tobj){
    ResetLegend(); Wpad->Clear(); Draw(Tobj);
  };

  /**
   * Prints a given plot type from a given collection.
   * If aFirst is set to false, the plot is printed on the existing print.
   * @param aPlotType plot type
   * @param aCollIndex collection index
   * @param aFirst flags first print
   */
  bool PrintPlot(const string aPlotType, const int aCollIndex=0, const bool aFirst=true);

  /**
   * Prints a processed plot.
   * If aFirst is set to false, the plot is printed on the existing print.
   * @param aPlotType plot type
   * @param aProcType process type
   * @param aCollIndexes collection indexes
   * @param aFirst flags first print
   */
  bool PrintProcPlot(const string aPlotType, const string aProcType, const string aProcName, const vector<int> aCollIndexes, const bool aFirst=true);

  /**
   * Prints a given plot type from a few collections.
   * If aFirst is set to false, the plots are printed on the existing print.
   * @param aPlotType plot type
   * @param aCollIndexes collection indexes
   * @param aFirst flags first print
   */
  bool PrintPlots(const string aPlotType, const vector<int> aCollIndexes, const bool aFirst=true);

  /**
   * Add an object to the current plot.
   */
  inline void AddObject(TObject *aTobj){ Draw(aTobj,"same"); }; 
  /**
   * Prints a given plot type from all collections.
   * The plots from all collections are superimposed on the same plot. The scale is set by the plot of the first collection.
   * @param aPlotType plot type
   */
  bool PrintCollectionPlot(const string aPlotType);

  /**
   * Prints all plots from all collections.
   * The plots from all collections are superimposed on the same plot. The scale is set by the plot of the first collection.
   */
  bool PrintCollectionPanel(void);

  /**
   * Returns the selection of a given collection.
   * The returned TriggerSelect pointer can be used to modify the selection of a given collection using the methods of the TriggerSelect class. NULL is returned if the call fails.
   *
   * After a new selection is defined, the plots need to be remade with MakeCollections() and printed with PrintCollectionPlot();
   * @param aCollIndex collection index
   */
  TriggerSelect* GetCollectionSelection(const int aCollIndex=0);

  /**
   * Sets the time format.
   * This function can be used to switch the time format from a GPS scale to a date format.
   * @param aDateFormat time format: true = date, false = GPS
   */
  void SetDateFormat(const bool aDateFormat=true);

  /**
   * Sets selection segments to all collections.
   * Collection triggers/clusters can be selected using a segment list. If the trigger/cluster peak time is not in the selection segment list, it is rejected. specific blue strips are plotted to indicate the selection segments.
   *
   * When calling this function with a pointer to NULL or an invalid segment list, no selection is performed.
   *
   * @warning The Segments object is NOT copied, only the pointer is. As a result, do not modify the segment list after making the collection.
   *
   * @param SelectionSegments Pointer to a valid segment list.
   */
  void SetSelectionSegments(Segments *SelectionSegments);

  /**
   * Draw the time box.
   * @param aTB the time box is drawn if this flag is set to true
   */
  inline void SetTimeBox(const bool aTB=true){ tb=aTB; };

  /**
   * Sets a legend to a collection.
   * Then use GwollumPlot::DrawLegend() to add the legend to the current plot.
   * @param aCollIndex collection index
   * @param aLegend legend label
   */
  bool SetCollectionLegend(const int aCollIndex, const string aLegend);

  /**
   * Assigns a color to a collection.
   * @param aCollIndex collection index
   * @param aColorIndex color index (ROOT conventions)
   */
  bool SetCollectionColor(const int aCollIndex, const int aColorIndex);

  /**
   * Assigns a marker style to a collection.
   * @param aCollIndex collection index
   * @param aMarkerStyleIndex marker style index (ROOT conventions)
   * @param aMarkerSizeIndex marker size index (ROOT conventions)
   */
  bool SetCollectionMarker(const int aCollIndex, const int aMarkerStyleIndex, const double aMarkerSizeIndex=1.0);


  /**
   * Use clusters for a given collection.
   * This function modifies the selection of collection 'aCollIndex': clusters are used instead of triggers. Of course the trigger set must be clustered first.
   *
   * In fact the selection is applied to the cluster tag set with Triggers::SetClusterTag() with the following rules:
   * - if 'aUseClusters < 0' --> triggers are used
   * - if 'aUseClusters = 0' --> clusters with tag>=0 are used
   * - if 'aUseClusters > 0' --> clusters with tag==aUseClusters are used
   * @param aCollIndex collection index
   * @param aUseClusters cluster tag selection
   */
  bool SetCollectionUseClusters(const int aCollIndex, const int aUseClusters=1);

  /**
   * Returns the number of clusters in a given collection.
   * @param aCollIndex collection index
   */
  int GetCollectionNClusters(const int aCollIndex);

  /**
   * Returns the total event time in a given collection [s].
   * @param aCollIndex collection index
   */
  double GetCollectionTotalEventDuration(const int aCollIndex);

  /**
   * Returns a TH1D plot copy from a given collection.
   * Returns a pointer to NULL if this function fails.
   * @param aPlotType plot type
   * @param aCollIndex collection index
   */
  TH1D* GetTH1D(const string aPlotType, const int aCollIndex);

  /**
   * Returns the Y-axis current maximum value.
   * @param aPlotType plot type
   * @param aCollIndex collection index
   */
  double GetYmax(const string aPlotType, const int aCollIndex);
  /**
   * Returns the Y-axis current minimum value.
   * @param aPlotType plot type
   * @param aCollIndex collection index
   */
  double GetYmin(const string aPlotType, const int aCollIndex);

  /**
   * Returns the number of collections.
   */
  inline int GetNCollections(void){ return Ncoll; };
  
  /**
   * Plots the loudest event with a star marker.
   * @param aFlag Set to true to plot a star.
   */
  inline void PlotLoudestEvent(const bool aFlag=true){ plot_star=aFlag; };
  

 protected:
  
  // PLOTS
  TH2D   **Hfreqtime;                 ///< freq vs. time container
  TGraph **Gfreqtime;                 ///< freq vs. time
  TGraph *Gfreqtimeloud;              ///< freq vs. time (loudest event)
  TGraph *Gfreqtimeloud2;             ///< freq vs. time (loudest event)
  TH2D   **Hsnrtime;                  ///< SNR vs. time container
  TGraph **Gsnrtime;                  ///< SNR vs. time
  TH2D   **Hsnrfreq;                  ///< SNR vs. freq container
  TGraph **Gsnrfreq;                  ///< SNR vs. freq
  TH1D   **Htime;                     ///< time (rate)
  TH1D   **Hfreq;                     ///< frequency
  TH1D   **Hsnr;                      ///< SNR
  TH1D   *Hproc;                      ///< processed histogram
 
  // COLLECTIONS
  int Ncoll;                          ///< number of collections

 private:

   // COLLECTIONS
  void ResetCollection(const int aCollIndex);

  // SELECTION
  void ResetParameters(const int aCollIndex);
  TriggerSelect **CollectionParam;    ///< collection selection
  int *Suseclusters;                  ///< use clusters flag
  double TPlivetime;                  ///< effective livetime
  double *ev_duration_tot;            ///< total event time
  Segments *SelSeg;                   ///< selection segments for all collections
  
  // STYLE
  void ApplyStyle(const int aCollIndex);
  int    *TPcolor;                    ///< TP colors
  double *TPmsize;                    ///< TP marker sizes
  int    *TPlsize;                    ///< TP line sizes
  int    *TPmstyle;                   ///< TP marker style
  string *Slegend;                    ///< selection legend
  bool Pusedate;                      ///< use date format flag
  bool plot_star;                     ///< flag to plot the loudest event
  bool tb;                            ///< flag to draw the time box
  TPave *SegBox;                      ///< time box
  TPave **OnBox;                      ///< ON segments
  TPave **SelBox;                     ///< Selected segments
  int nonbox;                         ///< number of on box
  int nselbox;                        ///< number of selected box

  ClassDef(TriggerPlot,0)  
};

#endif


