//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#ifndef __IO__
#define __IO__

#include "CUtils.h"

using namespace std;

/**
 * Parse option files.
 * This class was designed to read an option file and to extract parameters. Options are defined by a tag and a keyword.
 * \author    Florent Robinet
 */
class IO {

 private:
  
  vector< pair<string,string> > fAllLines;

  // parse one file
  bool ParseFile(const char* filename);
  
  // counter
  int fCurline;
  
  // current key 
  string fCurkey;
  
  // current tag 
  string fCurtag;
  
  // maximum number of characters in a line
  static const unsigned sLinesize;

 public:
  
  // builds IO from files matching filepattern 
  /**
   * @name Constructors and destructors
   @{
  */

  /**
   * Constructor of the IO class.
   * It loads and reads the option files listed in filepattern
   *
   * An option is defined by a single line:
   * \verbatim TAG  KEY  [value] \endverbatim
   * The [value] can be of any C++ type (string, integer...)
   *
   * It is also possible to have more than one value:
   * \verbatim TAG  KEY  [value1]   [value2]   [value3]   [value4] \endverbatim
   * In that case, values are stored in a vector container.
   *
   * Option lines can be commented out when lines start with the following characters: '//', '%' or '#'
   * @param filepattern file pattern pointing to the option files to be parsed.
   */
  IO(const char* filepattern);

  /**
   * Destructor of the IO class.
   */
  virtual ~IO(void) {fAllLines.clear();}
 
  /**
     @}
  */

 
  /**
   * Dumps all options.
   * By default it is dumped in the standard output.
   * @param out ouput stream
   */
  void Dump(ostream& out = cout) const;
  
  /**
   * Flags the parsing sequence.
   * This function returns false if the option files could not be parsed correctly.
   */
  inline bool IsZombie(void) const {return !fAllLines.size();}

  friend ostream& operator<<(ostream& out, IO& io);
  
  /**
   * Gets an option vector.
   * This template function fills a vector with values defined by a tag and a keyword.
   * false is returned if the option is not found.
   * @param tag tag name.
   * @param key keyword.
   * @param values returned vector.
  */
  template <class T>
    bool GetOpt(const char* tag, const char* key, vector< T >& values) const; 
  
  /**
   * Gets an option.
   * This template function gets an option value defined by a tag and a keyword.
   * false is returned if the option is not found.
   * @param tag tag name.
   * @param key keyword.
   * @param value returned value.
  */
  template <class T>
    bool GetOpt(const char* tag, const char* key, T& value) const; 
  
  /**
   * Gets an option of a string type.
   * This function gets an option string value defined by a tag and a keyword.
   * false is returned if the option is not found.
   * @param tag tag name.
   * @param key keyword.
   * @param value returned string value.
  */
  bool GetOpt(const char* tag, const char* key, string& value) const;
  
  /**
   * Returns the option line defined by a tag and a keyword.
   * If several lines match the tag and keyword, only the last line is retained.
   * @param tag tag name
   * @param key keyword
  */
  string GetLineData(const char* tag, const char* key) const;
  
  /**
   * Gets a vector of all options.    
   * This template function fills a vector with values defined by a tag and a keyword. This function gets all options matching a tag and a keyword. The combination of tag/keyword can be used over several lines.
   * false is returned if the option is not found.
   * @param tag tag name
   * @param key keyword
   * @param option value vector
   */
  template <class T>  
    bool GetAllOpt(const char* tag, const char* key, vector< T >& values);       
  /**
   * Gets options iteratively.
   * This function gets options matching a tag and a keyword. This function is useful when a combination of tag/keyword is repeated over several lines. At each call of this function, the next option value is returned.
   * false is returned if the option is not found.
   * @param tag tag name
   * @param key keyword
   * @param option value
   */
  template <class T>
    bool GetAllOpt(const char* tag, const char* key, T& value); 


  /**
   * Returns the next option line defined by a tag and a keyword.
   * This function can be called many times (in a row!) to read several lines defined by the same tag/keyword.
   * @param tag tag name
   * @param key keyword
  */
  string GetNextLineData(const char* tag, const char* key);

};

template <class T>
bool IO::GetOpt(const char* tag, const char* key, vector< T >& values) const {
  string data = GetLineData(tag,key);
 
  istringstream in(data.c_str());  
  while(1) {
    T tmp;
    in>>tmp;
    if(!in) break;
    values.push_back(tmp);
  } 
  
  return (bool)values.size();
}

template <class T>
bool IO::GetOpt(const char* tag, const char* key, T& value) const {
  string data = GetLineData(tag,key);
  
  istringstream in(data.c_str());  
  in>>value;
  return (!!in)&&data.size();
}

template <class T>
bool IO::GetAllOpt(const char* tag, const char* key, vector< T >& values) {
 
  string data;
  T tmp;
  while(1) {//  loop over lines
    data=GetNextLineData(tag, key);
    if(!data.compare("")) break;
    istringstream inval(data.c_str());  
    while(1) {// loop over values
      inval>>tmp;
      if(!inval) break;
      values.push_back(tmp);
    }
    inval.clear();
  } 

  return (bool)values.size();
}

template <class T>
bool IO::GetAllOpt(const char* tag, const char* key, T& value) {
  string data = GetNextLineData(tag, key);
  if(data.empty()) return false;
  
  istringstream in(data.c_str());  
  in>>value;
  if(in) {
    return true;
  }
  else {
    return false;  
  }
}

#endif
